# 5 February 2019
# Function for calculating the allocation based on the Coastal States (G16) Proposal
# this code takes in the variables pertaining to the coastal states catch allocation proposal, the catch history as outlined
# in that proposal, information on the IOTC members and which countries are sids, dcs, etc.
# 

#
make_COASTAL_allocation <- function(  simulation_spec , baselineDATA=baselineDATA_Final, IOTC_data = IOTC_data) { # 
  # store the baseline data 
  baselineDATA_static <- baselineDATA
  # initial overall allocations
  base_coastal_prop   <-simulation_spec[ ,"Coast"]
  base_HistCatch      <- simulation_spec[ , "Historical" ] # will simulate over the range of 0.50 - 0.70
  sup_high_seas       <-simulation_spec[ ,"SUPHS" ] # will simualate over the range of 0.025 - 0.07
  
  #stopifnot( sum(base_coastal_prop, base_HistCatch, sup_high_seas) ==1 ) 
  if( sum(base_coastal_prop, base_HistCatch, sup_high_seas )   != 1) 
    stop("Error: The sum of the base Coastal, Historical and Sup. High Seas must equal 1!")
  #-----------------------------------------------------
  
  #  DEFINE WEIGHITNG ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # i.	Coastal State CPCs: 
  cst_st_CPC    <-simulation_spec[ ,"CS_CPC_cmp" ] # 20-40% of the Baseline Coastal State allocation - which is base_coastal_prop
  #ii.	Developing Coastal State CPCs: 
  Dev_CS_CPC    <-simulation_spec[ ,"DCS_CPC_cmp" ]  # 30-70% of the Baseline Coastal State allocation; which is base_coastal_prop
  Dev_CS_1      <-simulation_spec[ ,"DCS_HDI_cmp" ]      #HDI status 30-40% of the Developing Coastal states allocation
  Dev_CS_2      <-simulation_spec[ ,"DCS_GNI_cmp" ]    # GNI status 30-40% of the Developing Coastal states 
  Dev_CS_3      <-simulation_spec[ ,"DCS_SIDS_cmp" ]     # SIDS status 30-70% of the the Developing Coastal states allocation
  
  #stopifnot(sum(Dev_CS_1,Dev_CS_2, Dev_CS_3 ) ==1 )
  if( sum(Dev_CS_1, Dev_CS_2, Dev_CS_3)   != 1) 
    stop("Error: The sum of the base HDI, GNI and SIDS must equal 1!")
  
  #iii.	EEZ proportion: 
  EEZprop <-  simulation_spec[ ,"EEZ_cmp" ]     # 10-20% of the Baseline Coastal State allocation; which is base_coastal_prop
  #  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
  #stopifnot(cst_st_CPC+ Dev_CS_CPC +  EEZprop == 1)
  if( sum(cst_st_CPC, Dev_CS_CPC, EEZprop)   != 1) 
    stop("Error: The sum of the base HDI, GNI and SIDS must equal 1!")
  
  #  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~        
  # need to put in an error checking here for the colnames? of baseline data
  #
  # housekeeping
  IOTC_data$flag <- as.character(IOTC_data[ ,"flag" ])
  # 
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~function starts~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Take data and make it long apply the baseline Historical catch 
  baselineDATA<-  gather(baselineDATA,  "HA_method", "HA_prop", 3:5)  %>% # this makes a vector of HA_method  - and gives the proportional amount of the historical catch. 
    mutate(   HA_prop =  base_HistCatch *   HA_prop ) 
  
  # this used to take 
  # add in the coastal states 
  CS_wts <- left_join( baselineDATA_static, IOTC_data[,c("flag", "Coastal_State_CPC") ], by = "flag" ) %>% 
    mutate(   p_contrib1 = ifelse( p_contrib1>0,1,0) ) %>%
    mutate(   p_contrib2 = ifelse( p_contrib2>0,1,0) ) %>%
    mutate(   p_contrib3 = ifelse( p_contrib3>0,1,0) ) %>%   
    group_by(SPECIES_CODE ) %>%
    mutate(  p_contrib1 = base_coastal_prop* cst_st_CPC *( p_contrib1* Coastal_State_CPC/ (sum(p_contrib1*Coastal_State_CPC) )) ) %>%
    mutate(  p_contrib2 = base_coastal_prop* cst_st_CPC *( p_contrib2* Coastal_State_CPC/ (sum(p_contrib2*Coastal_State_CPC) )) ) %>%   
    mutate(  p_contrib3 = base_coastal_prop* cst_st_CPC *( p_contrib3* Coastal_State_CPC/ (sum(p_contrib3*Coastal_State_CPC) )) ) %>% 
    dplyr::select(flag, SPECIES_CODE,  p_contrib1, p_contrib2, p_contrib3 )  %>%
    gather(., "HA_method", "CS_prop", 3:5) 
  
  # CS_wts[1:20,]
  # colSums(CS_wts[,3:5], na.rm=TRUE)
  #dim(CS_wts)
  
  # the issue here is that not all Coastal State CPC's have baseline historical catch for each species.....
  #sum(CS_wts$CS_prop) # so that is  5 species* basecoastalProp (0.25) * costal CPCallotment(0.2) * 3 historical allotments = 0.75 
  #    5* 0.25*0.2*3 =0.75
  
  #  head(CS_wts)
  #  sum(CS_wts$CS_CPC) 
  #  stopifnot(    sum(CS_wts$CS_CPC) == cst_st_CPC ) 
  #-----------------------------------------------------------
  #update data  with the coastal states weights
  baselineDATA2<-  baselineDATA %>%  left_join(.,CS_wts, by= c('flag', "SPECIES_CODE", "HA_method" ) )  # make the baseline Coastal state weighting1 
  
  #-----------------------------------------------------------
  
  #sum(baselineDATA2$CS_prop) #  = 0.75 # should be 3# 0.2 * 5 species * 3 historical catch allocation. 
  #----------------------------------------------------------------------------------------------------------------------
  #--~~ DCS indicators --~~ DCS indicators --~~ DCS indicators --~~ DCS indicators --~~ DCS indicators --~~ DCS indicators 
  # make the developing Coastal State   indicators, taking into account the base_coastal_prop
  # Note that not all species get caught by all countries  and don't get allocation based on different HA methods
  #----------------------------------------------------------------------------------------------------------------------       
  
  DCS_wts1 <-    left_join( baselineDATA_static,  IOTC_data[,c("flag", "HDI", "GNI", "SIDS") ] ,by = "flag" ) %>%
    mutate(   p_contrib1 = ifelse( p_contrib1>0,1,0) ) %>%
    mutate(   p_contrib2 = ifelse( p_contrib2>0,1,0) ) %>%
    mutate(   p_contrib3 = ifelse( p_contrib3>0,1,0) ) %>%   
    group_by(SPECIES_CODE ) %>%
    mutate(  p_contrib1 = base_coastal_prop* Dev_CS_CPC *Dev_CS_1*( p_contrib1* HDI/ (sum(p_contrib1*HDI) )) )%>%
    mutate(  p_contrib2 = base_coastal_prop* Dev_CS_CPC *Dev_CS_1*( p_contrib2* HDI/ (sum(p_contrib2*HDI) )) ) %>%   
    mutate(  p_contrib3 = base_coastal_prop* Dev_CS_CPC *Dev_CS_1*( p_contrib3* HDI/ (sum(p_contrib3*HDI) )) ) %>% 
    dplyr::select(flag, SPECIES_CODE,  p_contrib1, p_contrib2, p_contrib3 )  %>%
    gather(., "HA_method", "DCS_HDI_prop", 3:5) 
  
  DCS_wts2 <-    left_join( baselineDATA_static,  IOTC_data[,c("flag", "HDI", "GNI", "SIDS") ] ,by = "flag" ) %>%
    mutate(   p_contrib1 = ifelse( p_contrib1>0,1,0) ) %>%
    mutate(   p_contrib2 = ifelse( p_contrib2>0,1,0) ) %>%
    mutate(   p_contrib3 = ifelse( p_contrib3>0,1,0) ) %>%   
    group_by(SPECIES_CODE ) %>%
    mutate(  p_contrib1 = base_coastal_prop* Dev_CS_CPC *Dev_CS_2*( p_contrib1* GNI/ (sum(p_contrib1*GNI) )) )%>%
    mutate(  p_contrib2 = base_coastal_prop* Dev_CS_CPC *Dev_CS_2*( p_contrib2* GNI/ (sum(p_contrib2*GNI) )) ) %>%   
    mutate(  p_contrib3 = base_coastal_prop* Dev_CS_CPC *Dev_CS_2*( p_contrib3* GNI/ (sum(p_contrib3*GNI) )) ) %>% 
    dplyr::select(flag, SPECIES_CODE,  p_contrib1, p_contrib2, p_contrib3 )  %>%
    gather(., "HA_method", "DCS_GNI_prop", 3:5) 
  
  DCS_wts3 <-    left_join( baselineDATA_static,  IOTC_data[,c("flag", "HDI", "GNI", "SIDS") ] ,by = "flag" ) %>%
    mutate(   p_contrib1 = ifelse( p_contrib1>0,1,0) ) %>%
    mutate(   p_contrib2 = ifelse( p_contrib2>0,1,0) ) %>%
    mutate(   p_contrib3 = ifelse( p_contrib3>0,1,0) ) %>%   
    group_by(SPECIES_CODE ) %>%
    mutate(  p_contrib1 = base_coastal_prop* Dev_CS_CPC *Dev_CS_3*( p_contrib1* SIDS/ (sum(p_contrib1*SIDS) )) )%>%
    mutate(  p_contrib2 = base_coastal_prop* Dev_CS_CPC *Dev_CS_3*( p_contrib2* SIDS/ (sum(p_contrib2*SIDS) )) ) %>%   
    mutate(  p_contrib3 = base_coastal_prop* Dev_CS_CPC *Dev_CS_3*( p_contrib3* SIDS/ (sum(p_contrib3*SIDS) )) ) %>% 
    dplyr::select(flag, SPECIES_CODE,  p_contrib1, p_contrib2, p_contrib3 )  %>%
    gather(., "HA_method", "DCS_SIDS_prop", 3:5) 
  
  
  DCS_wts <- left_join(DCS_wts1, DCS_wts2, by = c("flag", "SPECIES_CODE", "HA_method")) %>% 
    left_join(., DCS_wts3, by = c("flag", "SPECIES_CODE", "HA_method"))
  #DCS_wts # 510 rows, 34 flags * 5 species # 
  # 
  
  #_____________________________________________________________________________________________
  # update the data file with the developing coastal states weights
  baselineDATA3<-   baselineDATA2 %>%  left_join(.,DCS_wts, by = c("flag", "SPECIES_CODE", "HA_method") )
  #_____________________________________________________________________________________________
  head(baselineDATA3)
  
  
  # Create weights for EEZ, taking into account the base_coastal_prop 
  
  EEZ_wts <- left_join( baselineDATA_static,  IOTC_data[,c("flag", "EEZ") ], by = "flag"  ) %>%
    mutate(   p_contrib1 = ifelse( p_contrib1>0,1,0) ) %>%
    mutate(   p_contrib2 = ifelse( p_contrib2>0,1,0) ) %>%
    mutate(   p_contrib3 = ifelse( p_contrib3>0,1,0) ) %>%   
    mutate(EEZ = replace_na(EEZ, 0)) %>% 
    group_by(SPECIES_CODE ) %>%
    mutate(  p_contrib1 = base_coastal_prop*  EEZprop*( p_contrib1* EEZ/ (sum(p_contrib1*EEZ) )) )%>%
    mutate(  p_contrib2 = base_coastal_prop*  EEZprop*( p_contrib2* EEZ/ (sum(p_contrib2*EEZ) )) ) %>%   
    mutate(  p_contrib3 = base_coastal_prop*  EEZprop*( p_contrib3* EEZ/ (sum(p_contrib3*EEZ) )) ) %>% 
    dplyr::select(flag, SPECIES_CODE,  p_contrib1, p_contrib2, p_contrib3 )  %>%
    gather(., "HA_method", "EEZ_prop", 3:5) 
  EEZ_wts
  
  #________________________________________________________________
  #_____________________________________________________________________________________________
  # update the data file  with the eez data 
  baselineDATA3<-   baselineDATA3 %>%  left_join(.,EEZ_wts, by  = c("flag", "SPECIES_CODE", "HA_method"))  
  class(baselineDATA3)
  #_____________________________________________________________________________________________
  
  # add in the supplementary high seas catch
  #  note this has a sunset provision
  # which I'll code up later. 
  #  
  SupHIGH_SEAS <- IOTC_data [,c("flag", "CP", "CNCP") ]    %>% mutate( CNCP = 0.5 * CNCP ) %>% 
    mutate(SUP_relative = CP+CNCP ) %>%
    mutate(    SUPHS_prop = sup_high_seas*(   SUP_relative / sum(SUP_relative) )  ) 
  
  SUPHS_wts <- left_join( baselineDATA_static,  SupHIGH_SEAS[,c("flag", "SUP_relative") ] , by = "flag" ) %>%
    mutate(   p_contrib1 = ifelse( p_contrib1>0,1,0) ) %>%
    mutate(   p_contrib2 = ifelse( p_contrib2>0,1,0) ) %>%
    mutate(   p_contrib3 = ifelse( p_contrib3>0,1,0) ) %>%   
    #  mutate(EEZ = replace_na(EEZ, 0)) %>% 
    group_by(SPECIES_CODE ) %>%
    mutate(  p_contrib1 = sup_high_seas * ( p_contrib1* SUP_relative/ (sum(p_contrib1*SUP_relative) )) )%>%
    mutate(  p_contrib2 = sup_high_seas * ( p_contrib2* SUP_relative/ (sum(p_contrib2*SUP_relative) )) ) %>%   
    mutate(  p_contrib3 = sup_high_seas * ( p_contrib3* SUP_relative/ (sum(p_contrib3*SUP_relative) )) ) %>% 
    dplyr:: select(flag, SPECIES_CODE,  p_contrib1, p_contrib2, p_contrib3 )  %>%
    gather(., "HA_method", "SUP_HS_prop", 3:5)
  sum(SUPHS_wts$SUP_HS_prop) # this = 0.75, which is 0.05*5 species * 3 historical catch allocations.  
  
  
  #_____________________________________________________________________________________________
  # update the data file  with the eez data 
  baselineDATA4<-   baselineDATA3 %>%  left_join(.,SUPHS_wts, by = c("flag", "SPECIES_CODE", "HA_method"))  
  class(baselineDATA4); as_tibble(baselineDATA4)
  sum(baselineDATA4$HA_prop) # this is 10.5, which means it does   have the historical allocation factored in  
  
  #_____________________________________________________________________________________________
  # calculate the overall proporiton of catch by species and country. 
  baselineDATA5<-   baselineDATA4 %>%   
    mutate( Country_Prop =  HA_prop  +  CS_prop +  DCS_HDI_prop + DCS_GNI_prop +  DCS_SIDS_prop + EEZ_prop + SUP_HS_prop ) # %>%
  # mutate( FTAC = Country_Prop * GTAC )
  
  # sum(baselineDATA5$Country_Prop) #   this should be 15.   5 species by 3 different Hisotrical catch allocations
  
  flag_species_prop <- baselineDATA5 %>% dplyr::select(flag, SPECIES_CODE,HA_method, Country_Prop)
  
  return(flag_species_prop )
} # end function

